/*
 * Licensed Materials - Property of IBM
 * 5648-C63
 * (C) Copyright IBM Corp. 2002
 */

package com.isv;

import java.io.*;
import com.ibm.broker.plugin.*;
import com.ibm.ctg.client.*;

/**
 * JavaCICSClient plugin node.
 * This node copies the contents of the input buffer (bitstream) into the CICS
 * commarea and sends an ECIRequest to the CICS server.
 * This node waits for the response from CICS syncronously.
 * The contents of the commarea will be propagated as a BLOB to the out terminal.
 * Any exception will terminate the process of the
 * message.
 */
public class JavaCICSClientNode extends MbNode implements MbNodeInterface
{
  // Node attributes
  String _regionName="";         //Mandatory
  String _programName="";        //Mandatory
  String _userid="";             //Optional
  String _password="";           //Optional
  String _transId="";            //Optional
  String _gatewayUrl="local://"; //Optional
  int _gatewayPort=2006;         //Optional
  int _commareaLength=0;         //Optional
  boolean _removeMQMD=true;      //Optional

  JavaGateway jgw = new JavaGateway();

  /**
   * JavaCICSClient node constructor.
   * This is where input and output terminal are created.
   */
  public JavaCICSClientNode() throws MbException
  {
    createInputTerminal("in");
    createOutputTerminal("out");
    createOutputTerminal("failure");
  }

  /**
   * This static method is called by the framework to identify this node.
   * If this method is not supplied, a default name will be generated
   * automatically based on the node's package/class name.  In this case
   * it would be 'ComIsvJavaCICSClientNode'.
   */
  public static String getNodeName()
  {
    return "ComIsvJavaCICSClientNode";
  }
  /**
   * This method writes messages to the WMQI system log and throws a user exception.
   */
  private void logErrorMessage( String className, String methodName, String traceText, Object [] inserts) 
    throws MbException{
	MbService ms = new MbService();
	ms.logError(className,
	            methodName,
	            "WMQIV210",
	            "2949",
	            traceText,
	            inserts);
    throw new MbUserException( className,
                               methodName,
                               "WMQIV210",
                               "2949",
                               traceText,
                               inserts );
  }

  /**
   * This evaluate message is called by the broker for each message passing
   * through the flow.  The message assembly is passed in with the 'assembly'
   * parameter.  It is possible for a node to have more than one input
   * terminal.  The terminal that the message has come in on is represented
   * by the 'in' parameter.
   */
  public void evaluate(MbMessageAssembly assembly, MbInputTerminal in)
    throws MbException
  {
 /* debug
    System.out.println("regionName: " + _regionName );  //debug
    System.out.println("programName: " + _programName );  //debug
    System.out.println("userid: " + _userid );  //debug
    System.out.println("password: " + _password );  //debug
    System.out.println("transId: " + _transId );  //debug
    System.out.println("commareaLength: " + _commareaLength );  //debug
    System.out.println("removeMQMD: " + _removeMQMD );  //debug
    System.out.println("gatewayUrl: " + _gatewayUrl );  //debug
    System.out.println("gatewayPort: " + _gatewayPort );  //debug
  */
    // Check that the connection to the CICS Transaction Gateway is still active
    try {
      if (!jgw.isOpen()) {
      // System.out.println("JavaGateway is closed" );  //debug
         jgw.setURL(_gatewayUrl);
         jgw.setPort(_gatewayPort);
         jgw.open();
      }

      // The incoming message assembly and its embedded messages are read-only.
      // New objects must be created using the copy constructors
      MbMessage newMsg = new MbMessage(assembly.getMessage());

      // If required remove the MQMD from the commarea to send to CICS
      if (_removeMQMD) {
         MbElement rootElement = newMsg.getRootElement();
         MbElement mqmdElement = rootElement.getFirstElementByPath("/MQMD");
         mqmdElement.detach();
      }
      byte[] commArea = new byte[_commareaLength];
      commArea = newMsg.getBuffer();

      // Prepare CICS request message
      ECIRequest ero =
         new ECIRequest(ECIRequest.ECI_SYNC        //ECI Call Type
                      ,_regionName                 //CICS Server applid
                      ,_userid                     //CICS Userid
                      ,_password                   //CICS Password
                      ,_programName                //CICS Program to be run
                      ,_transId                    //CICS Transaction to be run
                      ,commArea                    //CICS commarea
                      ,_commareaLength             //CICS commarea length
                      ,ECIRequest.ECI_NO_EXTEND    //ECI Extended mode
                      ,0);                         //ECI LUW Token

      int iRc = jgw.flow(ero);
      if (iRc != 0) {
 /* debug
         System.out.println("CICS RC: " + ero.getCicsRc() );  //debug
         System.out.println("Flow RC: " + iRc );      //debug
  */
         String inserts [] = new String []  {
                                        "CICS Interface Error ",
                                        "CICS RC: ",
                                        String.valueOf(ero.getCicsRcString()),
                                        " Flow RC: ",
                                        String.valueOf(iRc),
                                        "",
                                        "",
                                        "",
                                        "",
                                        "" };
         
         logErrorMessage("JavaGateway",
                          "flow",
                           String.valueOf(ero.getCicsRcString()),
                           inserts);   
         
      }
      // Prepare output message to propagate
      //    Get copy of original input message
      //    Detach the original input message text
      //    Create new message text from CICS commarea as a BLOB
      newMsg = new MbMessage(assembly.getMessage());
      MbElement rootElement = newMsg.getRootElement();
      MbElement messageElement = rootElement.getLastChild();
      messageElement.detach();                     // remove input message text
      messageElement = rootElement.createElementAsLastChild("NONE");     // MQMD Parser
      MbElement messageElement1 =
         messageElement.createElementAsFirstChild(MbElement.TYPE_NAME_VALUE,
                                                  "UnknownParserName",
                                                  "MQSTR");
      MbElement messageElement2 =
         messageElement1.createElementAfter(MbElement.TYPE_NAME_VALUE,
                                            "BLOB",
                                            commArea);

      MbMessageAssembly newAssembly = new MbMessageAssembly(assembly, newMsg);

      MbOutputTerminal out = getOutputTerminal("out");

      // Now propagate the message assembly.
      // If the terminal is not attached, an exception will be thrown.  The user
      // can choose to handle this exception, but it is not neccessary since
      // the framework will catch it and propagate the message to the failure
      // terminal, or if it not attached, rethrow the exception back upstream.
      out.propagate(newAssembly);

    } catch (MbException mbe) {
 //    mbe.printStackTrace();                             // debug 
       throw mbe;                                         //rethrow exception
    } catch (Exception e) {
 //   e.printStackTrace();      //debug

      String inserts [] = new String []  {
       	                  "CICS Interface error",
                           String.valueOf(e.getMessage()),
                           "",
                           "",
                           "",
                           "",
                           "",
                           "",
                           "",
                           "" };
                           
      logErrorMessage(getNodeName(), 
                       "evaluate", 
                       String.valueOf(e.getMessage()), 
                       inserts);
                     
     }
  }

  public String toString()
  {
    return getName();
  }

  /* Attributes are defined for a node by supplying get/set methods.
   * The following two methods define an attribute 'regionName'.
   * The capitalisation follows the usual JavaBean property convention.
   */
  public String getRegionName()
  {
    return _regionName;
  }

  public void setRegionName(String regionName)
  {
    _regionName = regionName;
  }
  /* Attributes are defined for a node by supplying get/set methods.
   * The following two methods define an attribute 'programName'.
   * The capitalisation follows the usual JavaBean property convention.
   */
  public String getProgramName()
  {
    return _programName;
  }

  public void setProgramName(String programName)
  {
    _programName = programName;
  }
  /* Attributes are defined for a node by supplying get/set methods.
   * The following two methods define an attribute 'userid'.
   * The capitalisation follows the usual JavaBean property convention.
   */
  public String getUserid()
  {
    return _userid;
  }

  public void setUserid(String userid)
  {
     if (userid == "") {
        _userid = null;
     } else {
     _userid = userid;
     }
  }
  /* Attributes are defined for a node by supplying get/set methods.
   * The following two methods define an attribute 'password'.
   * The capitalisation follows the usual JavaBean property convention.
   */
  public String getPassword()
  {
    return _password;
  }

  public void setPassword(String password)
  {
     if (password == "") {
        _password = null;
     } else {
     _password = password;
     }
  }
  /* Attributes are defined for a node by supplying get/set methods.
   * The following two methods define an attribute 'transId'.
   * The capitalisation follows the usual JavaBean property convention.
   */
  public String getTransId()
  {
    return _transId;
  }

  public void setTransId(String transId)
  {
     if (transId == "") {
        _transId = null;
     } else {
     _transId = transId;
     }
  }
  /* Attributes are defined for a node by supplying get/set methods.
   * The following two methods define an attribute 'commareaLength'.
   * The capitalisation follows the usual JavaBean property convention.
   */
  public String getCommareaLength()
  {
    return String.valueOf(_commareaLength);
  }

  public void setCommareaLength(String commareaLength)
  {
    _commareaLength = Integer.parseInt(commareaLength);
  }
  /* Attributes are defined for a node by supplying get/set methods.
   * The following two methods define an attribute 'removeMQMD'.
   * The capitalisation follows the usual JavaBean property convention.
   */
  public String getRemoveMQMD()
  {
    return String.valueOf(_removeMQMD);
  }
  public void setRemoveMQMD(String removeMQMD)
  {
     if (removeMQMD.equals("true")) {
       _removeMQMD = true;
     } else {
       _removeMQMD = false;
     }
  }
  /* Attributes are defined for a node by supplying get/set methods.
   * The following two methods define an attribute 'gatewayUrl'.
   * The capitalisation follows the usual JavaBean property convention.
   */
  public String getGatewayUrl()
  {
    return _gatewayUrl;
  }

  public void setGatewayUrl(String gatewayUrl)
  {
    _gatewayUrl = gatewayUrl;
  }
  /* Attributes are defined for a node by supplying get/set methods.
   * The following two methods define an attribute 'gatewayPort'.
   * The capitalisation follows the usual JavaBean property convention.
   */
  public String getGatewayPort()
  {
    return String.valueOf(_gatewayPort);
  }

  public void setGatewayPort(String gatewayPort)
  {
    _gatewayPort = Integer.parseInt(gatewayPort);
  }
}
